@@ -3,19 +3,8 @@
 """
 Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-Kajongg is free software you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation either version 2 of the License, or
-(at your option) any later version.
+SPDX-License-Identifier: GPL-2.0
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 import weakref
@@ -24,7 +13,7 @@ from collections import defaultdict
 from log import logException, logWarning
 from mi18n import i18n, i18nc, i18nE
 from common import IntDict, Debug
-from common import StrMixin
+from common import StrMixin, Internal
 from wind import East
 from query import Query
 from tile import Tile, TileList, elements
@@ -33,7 +22,7 @@ from meld import Meld, MeldList
 from permutations import Permutations
 from message import Message
 from hand import Hand
-from intelligence import AIDefault
+from intelligence import AIDefaultAI
 
 
 class Players(list, StrMixin):
@@ -59,7 +48,7 @@ class Players(list, StrMixin):
         return list.__getitem__(self, index)
 
     def __str__(self):
-        return ', '.join(list('%s: %s' % (x.name, x.wind) for x in self))
+        return ', '.join('%s: %s' % (x.name, x.wind) for x in self)
 
     def byId(self, playerid):
         """lookup the player by id"""
@@ -67,6 +56,7 @@ class Players(list, StrMixin):
             if player.nameid == playerid:
                 return player
         logException("no player has id %d" % playerid)
+        return None
 
     def byName(self, playerName):
         """lookup the player by name"""
@@ -76,6 +66,7 @@ class Players(list, StrMixin):
         logException(
             "no player has name %s - we have %s" %
             (playerName, [x.name for x in self]))
+        return None
 
     @staticmethod
     def load():
@@ -91,6 +82,14 @@ class Players(list, StrMixin):
     @staticmethod
     def createIfUnknown(name):
         """create player in database if not there yet"""
+        if not Internal.db:
+            # kajonggtest
+            nameid = len(Players.allIds) + 1
+            Players.allIds[name] = nameid
+            Players.allNames[nameid] = name
+            if not name.startswith('Robot'):
+                Players.humanNames[nameid] = name
+
         if name not in Players.allNames.values():
             Players.load()  # maybe somebody else already added it
             if name not in Players.allNames.values():
@@ -102,8 +101,8 @@ class Players(list, StrMixin):
     def translatePlayerNames(self, names):
         """for a list of names, translates those names which are english
         player names into the local language"""
-        known = set(x.name for x in self)
-        return list(self.byName(x).localName if x in known else x for x in names)
+        known = {x.name for x in self}
+        return [self.byName(x).localName if x in known else x for x in names]
 
 
 class Player(StrMixin):
@@ -137,12 +136,12 @@ class Player(StrMixin):
         Players.createIfUnknown(name)
         self.name = name
         self.wind = East
-        self.intelligence = AIDefault(self)
+        self.intelligence = AIDefaultAI(self)
         self.visibleTiles = IntDict(game.visibleTiles) if game else IntDict()
         self.handCache = {}
         self.cacheHits = 0
         self.cacheMisses = 0
-        self.__lastSource = TileSource.East14th
+        self.__lastSource = TileSource.Unknown
         self.clearHand()
         self.handBoard = None
 
@@ -154,7 +153,7 @@ class Player(StrMixin):
 
     def clearCache(self):
         """clears the cache with Hands"""
-        if Debug.hand and len(self.handCache):
+        if Debug.hand and self.handCache:
             self.game.debug(
                 '%s: cache hits:%d misses:%d' %
                 (self, self.cacheHits, self.cacheMisses))
@@ -183,8 +182,7 @@ class Player(StrMixin):
     @property
     def game(self):
         """hide the fact that this is a weakref"""
-        if self._game:
-            return self._game()
+        return self._game() if self._game else None
 
     def clearHand(self):
         """clear player attributes concerning the current hand"""
@@ -197,7 +195,7 @@ class Player(StrMixin):
         self.newHandContent = None
         self.originalCallingHand = None
         self.__lastTile = None
-        self.lastSource = TileSource.East14th
+        self.lastSource = TileSource.Unknown
         self.lastMeld = Meld()
         self.__mayWin = True
         self.__payment = 0
@@ -220,6 +218,8 @@ class Player(StrMixin):
         """temp for debugging"""
         assert isinstance(value, (Tile, type(None))), value
         self.__lastTile = value
+        if value is None:
+            self.lastMeld = Meld()
 
     def invalidateHand(self):
         """some source for the computation of current hand changed"""
@@ -296,10 +296,7 @@ class Player(StrMixin):
     @property
     def handTotal(self):
         """the hand total of this player for the final scoring"""
-        if not self.game.winner:
-            return 0
-        else:
-            return self.hand.total()
+        return 0 if not self.game.winner else self.hand.total()
 
     @property
     def balance(self):
@@ -354,16 +351,16 @@ class Player(StrMixin):
         else:
             try:
                 self._concealedTiles.remove(tile)
-            except ValueError:
+            except ValueError as _:
                 raise Exception('removeTile(%s): tile not in concealed %s' %
-                                (tile, ''.join(self._concealedTiles)))
+                                (tile, ''.join(self._concealedTiles))) from _
         if tile is self.lastTile:
             self.lastTile = None
         self._hand = None
 
     def addConcealedTiles(self, tiles, animated=False):  # pylint: disable=unused-argument
         """add to my tiles"""
-        assert len(tiles)
+        assert tiles
         for tile in tiles:
             if tile.isBonus:
                 self._bonusTiles.append(tile)
@@ -374,29 +371,37 @@ class Player(StrMixin):
 
     def syncHandBoard(self, adding=None):
         """virtual: synchronize display"""
-        pass
 
     def colorizeName(self):
         """virtual: colorize Name on wall"""
-        pass
 
-    def getsFocus(self, dummyResults=None):
+    def getsFocus(self, unusedResults=None):
         """virtual: player gets focus on his hand"""
-        pass
 
     def mjString(self):
         """compile hand info into a string as needed by the scoring engine"""
         announcements = 'a' if self.originalCall else ''
         return ''.join(['m', self.lastSource.char, ''.join(announcements)])
 
-    def makeTileKnown(self, tileName):
+    def makeTileKnown(self, tile):
         """used when somebody else discards a tile"""
         assert not self._concealedTiles[0].isKnown
-        self._concealedTiles[0] = tileName
+        self._concealedTiles[0] = tile
         self._hand = None
 
+    def __computeLastInfo(self):
+        """compile info about last tile and last meld into a list of strings"""
+        result = []
+        if self.lastTile:
+# TODO assert, dass lastTile in _concealedTiles oder in _exposedMelds ist
+# and (self.lastTile in self._concealedTiles or self.lastTile in :
+            result.append(
+                'L%s%s' %
+                (self.lastTile, self.lastMeld if self.lastMeld else ''))
+        return result
+
     def __computeHand(self):
-        """returns Hand for this player"""
+        """return Hand for this player"""
         assert not (self._concealedMelds and self._concealedTiles)
         melds = list()
         melds.extend(str(x) for x in self._exposedMelds)
@@ -405,10 +410,7 @@ class Player(StrMixin):
             melds.append('R' + ''.join(str(x) for x in sorted(self._concealedTiles)))
         melds.extend(str(x) for x in self._bonusTiles)
         melds.append(self.mjString())
-        if self.lastTile:
-            melds.append(
-                'L%s%s' %
-                (self.lastTile, self.lastMeld if self.lastMeld else ''))
+        melds.extend(self.__computeLastInfo())
         return Hand(self, ' '.join(melds))
 
     def _computeHandWithDiscard(self, discard):
@@ -485,15 +487,12 @@ class PlayingPlayer(Player):
 
     def popupMsg(self, msg):
         """virtual: show popup on display"""
-        pass
 
     def hidePopup(self):
         """virtual: hide popup on display"""
-        pass
 
     def speak(self, txt):
         """only a visible playing player can speak"""
-        pass
 
     def declaredMahJongg(self, concealed, withDiscard, lastTile, lastMeld):
         """player declared mah jongg. Determine last meld, show concealed tiles grouped to melds"""
@@ -531,7 +530,7 @@ class PlayingPlayer(Player):
             self._hand = None
 
     def __possibleChows(self):
-        """returns a unique list of lists with possible claimable chow combinations"""
+        """return a unique list of lists with possible claimable chow combinations"""
         if self.game.lastDiscard is None:
             return []
         exposedChows = [x for x in self._exposedMelds if x.isChow]
@@ -543,11 +542,11 @@ class PlayingPlayer(Player):
         return within.hasChows(tile)
 
     def __possibleKongs(self):
-        """returns a unique list of lists with possible kong combinations"""
+        """return a unique list of lists with possible kong combinations"""
         kongs = []
         if self == self.game.activePlayer:
             # declaring a kong
-            for tileName in sorted(set([x for x in self._concealedTiles if not x.isBonus])):
+            for tileName in sorted({x for x in self._concealedTiles if not x.isBonus}):
                 if self._concealedTiles.count(tileName) == 4:
                     kongs.append(tileName.kong)
                 elif self._concealedTiles.count(tileName) == 1 and \
@@ -563,28 +562,28 @@ class PlayingPlayer(Player):
                 kongs.append(Meld(discardTile * 4))
         return kongs
 
-    def __maySayChow(self, dummyMove):
-        """returns answer arguments for the server if calling chow is possible.
+    def __maySayChow(self, unusedMove):
+        """return answer arguments for the server if calling chow is possible.
         returns the meld to be completed"""
-        if self == self.game.nextPlayer():
-            return self.__possibleChows()
+        return self.__possibleChows() if self == self.game.nextPlayer() else None
 
-    def __maySayPung(self, dummyMove):
-        """returns answer arguments for the server if calling pung is possible.
+    def __maySayPung(self, unusedMove):
+        """return answer arguments for the server if calling pung is possible.
         returns the meld to be completed"""
         lastDiscard = self.game.lastDiscard
         if self.game.lastDiscard:
             assert lastDiscard.isConcealed, lastDiscard
             if self.concealedTiles.count(lastDiscard) >= 2:
                 return MeldList([lastDiscard.pung])
+        return None
 
-    def __maySayKong(self, dummyMove):
-        """returns answer arguments for the server if calling or declaring kong is possible.
+    def __maySayKong(self, unusedMove):
+        """return answer arguments for the server if calling or declaring kong is possible.
         returns the meld to be completed or to be declared"""
         return self.__possibleKongs()
 
     def __maySayMahjongg(self, move):
-        """returns answer arguments for the server if calling or declaring Mah Jongg is possible"""
+        """return answer arguments for the server if calling or declaring Mah Jongg is possible"""
         game = self.game
         if move.message == Message.DeclaredKong:
             withDiscard = move.meld[0].concealed
@@ -603,9 +602,10 @@ class PlayingPlayer(Player):
                     self, list(x for x in game.players), game.activePlayer))
                 game.debug('  with hand {}'.format(hand))
             return MeldList(x for x in hand.melds if not x.isDeclared), withDiscard, hand.lastMeld
+        return None
 
-    def __maySayOriginalCall(self, dummyMove):
-        """returns True if Original Call is possible"""
+    def __maySayOriginalCall(self, unusedMove):
+        """return True if Original Call is possible"""
         for tileName in sorted(set(self.concealedTiles)):
             newHand = self.hand - tileName
             if newHand.callingHands:
@@ -614,6 +614,7 @@ class PlayingPlayer(Player):
                         '%s may say Original Call by discarding %s from %s' %
                         (self, tileName, self.hand))
                 return True
+        return False
 
     sayables = {
         Message.Pung: __maySayPung,
@@ -637,8 +638,7 @@ class PlayingPlayer(Player):
         where a meld is represented by a list of 2char strings"""
         if msg in (Message.Chow, Message.Pung, Message.Kong):
             return [x for x in self.sayable[msg] if self.mustPlayDangerous(x)]
-        else:
-            return []
+        return []
 
     def hasConcealedTiles(self, tiles, within=None):
         """do I have those concealed tiles?"""
@@ -696,6 +696,7 @@ class PlayingPlayer(Player):
                 '%1 claiming MahJongg: She did not pass all concealed tiles to the server')
             return msg, self.name
         self._hand = None
+        return None
 
     def robTileFrom(self, tile):
         """used for robbing the kong from this player"""
@@ -744,7 +745,7 @@ class PlayingPlayer(Player):
             # declaring a kong is never dangerous because we get
             # an unknown replacement
             return False
-        afterExposed = list(x.exposed for x in self._concealedTiles)
+        afterExposed = [x.exposed for x in self._concealedTiles]
         if exposing:
             exposing = exposing[:]
             if self.game.lastDiscard:
@@ -809,7 +810,7 @@ class PlayingPlayer(Player):
             assert group.islower(), self.visibleTiles
             if group in Tile.colors:
                 if all(x.group == group for x in self.visibleTiles):
-                    suitTiles = set([Tile(group, x) for x in Tile.numbers])
+                    suitTiles = {Tile(group, x) for x in Tile.numbers}
                     if self.visibleTiles.count(suitTiles) >= 9:
                         dangerous.append(
                             (suitTiles, i18n('Player %1 may try a True Color Game', pName)))
@@ -827,11 +828,11 @@ class PlayingPlayer(Player):
             dragonsDangerous = dragonsDangerous or dragonMelds >= 2
             if windsDangerous:
                 dangerous.append(
-                    (set(x for x in elements.winds if x not in self.visibleTiles),
+                    ({x for x in elements.winds if x not in self.visibleTiles},
                      i18n('Player %1 exposed many winds', pName)))
             if dragonsDangerous:
                 dangerous.append(
-                    (set(x for x in elements.dragons if x not in self.visibleTiles),
+                    ({x for x in elements.dragons if x not in self.visibleTiles},
                      i18n('Player %1 exposed many dragons', pName)))
         self.dangerousTiles = dangerous
         if dangerous and Debug.dangerousGame:
