@@ -3,19 +3,8 @@
 """
 Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-Kajongg is free software you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation either version 2 of the License, or
-(at your option) any later version.
+SPDX-License-Identifier: GPL-2.0
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 # pylint: disable=ungrouped-imports
@@ -99,13 +88,12 @@ class ScorePlayerItem(ScoreTreeItem):
         """return the content stored in this node"""
         if column == 0:
             return i18n(self.rawContent[0])
-        else:
-            try:
-                return self.hands()[column - 1]
-            except IndexError:
-                # we have a penalty but no hand yet. Should
-                # not happen in practical use
-                return None
+        try:
+            return self.hands()[column - 1]
+        except IndexError:
+            # we have a penalty but no hand yet. Should
+            # not happen in practical use
+            return None
 
     def hands(self):
         """a small helper"""
@@ -167,7 +155,7 @@ class ScoreItemDelegate(QStyledItemDelegate):
                         # have to draw the lines into one big pixmap and copy
                         # from the into the cells
                         painter.drawPolyline(*chart)
-            return
+            return None
         return QStyledItemDelegate.paint(self, painter, option, index)
 
 
@@ -177,28 +165,28 @@ class ScoreModel(TreeModel):
     steps = 30  # how fine do we want the stepping in the chart spline
 
     def __init__(self, parent=None):
-        super(ScoreModel, self).__init__(parent)
+        super().__init__(parent)
         self.scoreTable = parent
         self.rootItem = ScoreRootItem(None)
         self.minY = self.maxY = None
         self.loadData()
 
     def chart(self, rect, index, playerItem):
-        """returns list(QPointF) for a player in a specific tree cell"""
+        """return list(QPointF) for a player in a specific tree cell"""
         chartHeight = float(rect.height()) * 4
         yScale = chartHeight / (self.minY - self.maxY)
         yOffset = rect.height() * index.row()
-        yValues = list(playerItem.chartPoints(index.column(), self.steps))
-        yValues = [(y - self.maxY) * yScale - yOffset for y in yValues]
+        _ = (playerItem.chartPoints(index.column(), self.steps))
+        yValues = [(y - self.maxY) * yScale - yOffset for y in _]
         stepX = float(rect.width()) / self.steps
-        xValues = list(x * stepX for x in range(self.steps + 1))
-        return list(QPointF(x, y) for x, y in zip(xValues, yValues))
+        xValues = [x * stepX for x in range(self.steps + 1)]
+        return [QPointF(x, y) for x, y in zip(xValues, yValues)]
 
     def data(self, index, role=None):  # pylint: disable=no-self-use,too-many-branches
         """score table"""
         # pylint: disable=too-many-return-statements
         if not index.isValid():
-            return
+            return None
         column = index.column()
         item = index.internalPointer()
         if role is None:
@@ -216,16 +204,9 @@ class ScoreModel(TreeModel):
                     else:
                         content = str(content.balance)
                 return content
-            else:
-                if column > 0:
-                    return ''
-                else:
-                    return item.content(0)
+            return '' if column > 0 else item.content(0)
         if role == Qt.TextAlignmentRole:
-            if index.column() == 0:
-                return int(Qt.AlignLeft | Qt.AlignVCenter)
-            else:
-                return int(Qt.AlignRight | Qt.AlignVCenter)
+            return int(Qt.AlignLeft | Qt.AlignVCenter) if index.column() == 0 else int(Qt.AlignRight | Qt.AlignVCenter)
         if role == Qt.FontRole:
             return QFont('Monospaced')
         if role == Qt.ForegroundRole:
@@ -244,6 +225,7 @@ class ScoreModel(TreeModel):
                 englishHints = content.manualrules.split('||')
                 tooltip = '<br />'.join(i18n(x) for x in englishHints)
                 return tooltip
+        return None
 
     def headerData(self, section, orientation, role):
         """tell the view about the wanted headers"""
@@ -258,10 +240,8 @@ class ScoreModel(TreeModel):
                 if not handResult.penalty:
                     return handResult.handId()
         elif role == Qt.TextAlignmentRole:
-            if section == 0:
-                return int(Qt.AlignLeft | Qt.AlignVCenter)
-            else:
-                return int(Qt.AlignRight | Qt.AlignVCenter)
+            return int(Qt.AlignLeft | Qt.AlignVCenter) if section == 0 else int(Qt.AlignRight | Qt.AlignVCenter)
+        return None
 
     def loadData(self):
         """loads all data from the data base into a 2D matrix formatted like the wanted tree"""
@@ -274,8 +254,8 @@ class ScoreModel(TreeModel):
             (x for x in game.players if not x.name.startswith('Robot')))
         robots = sorted(
             (x for x in game.players if x.name.startswith('Robot')))
-        data = list(tuple([player.localName, [HandResult(*x[1:]) for x in records
-                                              if x[0] == player.nameid]]) for player in humans + robots)
+        data = [tuple([player.localName, [HandResult(*x[1:]) for x in records
+                                          if x[0] == player.nameid]]) for player in humans + robots]
         self.__findMinMaxChartPoints(data)
         parent = QModelIndex()
         groupIndex = self.index(self.rootItem.childCount(), 0, parent)
@@ -334,8 +314,8 @@ class HandResult:
     def roundHand(self, allHands):
         """the nth hand in the current round, starting with 1"""
         idx = allHands.index(self)
-        allHands = list(reversed(allHands[:idx]))
-        allHands = list(x for x in allHands if not x.penalty)
+        _ = reversed(allHands[:idx])
+        allHands = [x for x in _ if not x.penalty]
         if not allHands:
             return 1
         for idx, hand in enumerate(allHands):
@@ -400,11 +380,11 @@ class HorizontalScrollBar(QScrollBar):
         QScrollBar.__init__(self, parent)
         self.scoreTable = scoreTable
 
-    def showEvent(self, dummyEvent):
+    def showEvent(self, unusedEvent):
         """adjust the left view"""
         self.scoreTable.adaptLeftViewHeight()
 
-    def hideEvent(self, dummyEvent):
+    def hideEvent(self, unusedEvent):
         """adjust the left view"""
         self.scoreTable.viewRight.header().setOffset(
             0)  # we should not have to do this...
@@ -421,7 +401,7 @@ class ScoreTable(QWidget):
     finished. To achieve this we keep our own reference to game."""
 
     def __init__(self, scene):
-        super(ScoreTable, self).__init__(None)
+        super().__init__(None)
         self.setObjectName('ScoreTable')
         self.scene = scene
         self.scoreModel = None
@@ -477,12 +457,12 @@ class ScoreTable(QWidget):
     def sizeHint(self):
         """give the scoring table window a sensible default size"""
         result = QWidget.sizeHint(self)
-        result.setWidth(result.height() * 3 / 2)
+        result.setWidth(result.height() * 3 // 2)
         # the default is too small. Use at least 2/5 of screen height and 1/4
         # of screen width:
-        available = Internal.app.desktop().availableGeometry()
-        height = max(result.height(), available.height() * 2 / 5)
-        width = max(result.width(), available.width() / 4)
+        available = KApplication.desktopSize()
+        height = max(result.height(), available.height() * 2 // 5)
+        width = max(result.width(), available.width() // 4)
         result.setHeight(height)
         result.setWidth(width)
         return result
@@ -545,7 +525,7 @@ class ScoreTable(QWidget):
         scrollBar = self.viewRight.horizontalScrollBar()
         scrollBar.setValue(scrollBar.maximum())
 
-    def showEvent(self, dummyEvent):
+    def showEvent(self, unusedEvent):
         """Only now the views and scrollbars have useful sizes, so we can compute the spacer
         for the left view"""
         self.adaptLeftViewHeight()
@@ -564,6 +544,10 @@ class ScoreTable(QWidget):
         if height:
             self.leftLayout.addSpacing(height)
 
+    def closeEvent(self, unusedEvent):  # pylint: disable=no-self-use
+        """update action button state"""
+        Internal.mainWindow.actionScoreTable.setChecked(False)
+
 
 class ExplainView(QListView):
 
@@ -615,6 +599,10 @@ class ExplainView(QListView):
             # QStringListModel does not optimize identical lists away, so we do
             self.model.setStringList(lines)
 
+    def closeEvent(self, unusedEvent):  # pylint: disable=no-self-use
+        """update action button state"""
+        Internal.mainWindow.actionExplain.setChecked(False)
+
 
 class PenaltyBox(QSpinBox):
 
@@ -677,7 +665,7 @@ class PenaltyDialog(QDialog):
         grid = QGridLayout(self)
         lblOffense = QLabel(i18n('Offense:'))
         crimes = list(
-            [x for x in game.ruleset.penaltyRules if not ('absolute' in x.options and game.winner)])
+            x for x in game.ruleset.penaltyRules if not ('absolute' in x.options and game.winner))
         self.cbCrime = ListComboBox(crimes)
         lblOffense.setBuddy(self.cbCrime)
         grid.addWidget(lblOffense, 0, 0)
@@ -879,6 +867,11 @@ class ScoringDialog(QWidget):
         pGrid.addWidget(self.lblLastMeld, 8, 0, 1, 2)
         pGrid.addWidget(self.cbLastMeld, 8, 2, 1, 2)
 
+        self.lblLastTile.setVisible(False)
+        self.cbLastTile.setVisible(False)
+        self.cbLastMeld.setVisible(False)
+        self.lblLastMeld.setVisible(False)
+
     def setupUiForPlayer(self, pGrid, idx):
         """setup UI elements for a player"""
         self.spValues[idx] = QSpinBox()
@@ -955,10 +948,11 @@ class ScoringDialog(QWidget):
         self.slotInputChanged()
 
     def computeLastTile(self):
-        """returns the currently selected last tile"""
+        """return the currently selected last tile"""
         idx = self.cbLastTile.currentIndex()
         if idx >= 0:
             return self.cbLastTile.itemData(idx)
+        return None
 
     def clickedPlayerIdx(self, checkbox):
         """the player whose box has been clicked"""
@@ -966,6 +960,7 @@ class ScoringDialog(QWidget):
             if checkbox == self.wonBoxes[idx]:
                 return idx
         assert False
+        return None
 
     def wonChanged(self):
         """if a new winner has been defined, uncheck any previous winner"""
@@ -982,6 +977,10 @@ class ScoringDialog(QWidget):
                 self.wonBoxes[idx].setChecked(False)
         if newWinner:
             self.draw.setChecked(False)
+            self.lblLastTile.setVisible(True)
+            self.cbLastTile.setVisible(True)
+        self.lblLastMeld.setVisible(False)
+        self.cbLastMeld.setVisible(False)
         self.fillLastTileCombo()
         self.slotInputChanged()
 
@@ -1000,10 +999,8 @@ class ScoringDialog(QWidget):
             newState = bool(self.game.winner.handBoard.uiTiles)
         except AttributeError:
             newState = False
-        self.lblLastTile.setEnabled(newState)
-        self.cbLastTile.setEnabled(newState)
-        self.lblLastMeld.setEnabled(newState)
-        self.cbLastMeld.setEnabled(newState)
+        self.lblLastTile.setVisible(newState)
+        self.cbLastTile.setVisible(newState)
         if self.game:
             for player in self.game.players:
                 player.refreshManualRules(self.sender())
@@ -1206,11 +1203,13 @@ class ScoringDialog(QWidget):
             lastTile = Internal.scene.computeLastTile()
             winnerMelds = [m for m in self.game.winner.hand.melds if len(m) < 4
                            and lastTile in m]
-            assert len(winnerMelds), 'lastTile %s missing in %s' % (
+            assert winnerMelds, 'lastTile %s missing in %s' % (
                 lastTile, self.game.winner.hand.melds)
             if len(winnerMelds) == 1:
                 self.cbLastMeld.addItem(QIcon(), '', str(winnerMelds[0]))
                 self.cbLastMeld.setCurrentIndex(0)
+                self.lblLastMeld.setVisible(False)
+                self.cbLastMeld.setVisible(False)
                 return
             showCombo = True
             self.__fillLastMeldComboWith(winnerMelds, indexedMeld, lastTile)
